; ==============================================================================
; Blitz3D GeoWall Game Engine Template
; (C) 2004 Jason Leigh, Electronic Visualization Laboratory
;
; Use this to get started on your Blitz3D GeoWall Game
; This code shows:
;	1. how to set up GeoWall stereo
;	2. how to create a game loop that will keep constant update rate on CPUs with different speeds
;	3. loads a simple animated test object
;
; Keys:
;	1 = mono
;	2 = stereo
;	3 = reverse stereo
;	+ = increase stereo separation
;	- = decrease sterep separation
;
; Created: 7/15/2004
; Version: 8/17/2004
; For comments email: spiff@evl.uic.edu
; -----------------------------------------------------------------------------

; Include geowall stereo library
Include "stereo.bb"
Include "global.bb"
Include "text.bb"
Include "sound.bb"
Include "particle.bb"
Include "player.bb"
Include "course.bb"
Include "title.bb"
Include "level.bb"
Include "crash.bb"
Include "game.bb"

; ==============================================================================
; Init graphics and stereo cameras
Global g_screenXSize = 1024
Global g_screenYSize = 768

; Stereo viewing flag: 0=stereo 1=mono
Global g_viewMode=0

Print "Choose Display Mode:"
Print "1 - Standard GeoWall Stereo (2048x768)"
Print "2 - Desktop                 (1280x1024)"
Print "3 - Desktop                 (1024x768)"
Print "4 - Desktop Stereo          (1024x384)"

choice=WaitKey()

Select choice

Case 49
	g_screenXSize=2048
	g_screenYSize=768
	g_viewMode=0
	
Case 50
	g_screenXSize=1280
	g_screenYSize=1024
	g_viewMode=1

Case 52
	g_screenXSize=1024
	g_screenYSize=384
	g_viewMode=0

Default
	g_screenXSize=1024
	g_screenYSize=768
	g_viewMode=1
End Select

Graphics3D g_screenXSize, g_screenYSize

SetBuffer BackBuffer() 

; This is the main camera that you control in the game.
Global g_mainCamera = SL_CreateStereoCameras()
SL_SetUpCameras()

If g_viewMode = 1 SL_SwitchToMono()
		
; ==============================================================================

; Eye separation (-ve means cross-eyed stereo) 
Global g_eyeSeparation#=0.2
Global g_reverseStereo#=1
SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)

; ==============================================================================
; Set the game update/calculation rate to 30 frames per second.
; This is NOT the same as FRAME RATE which marks how rapidly the graphics image is updating.
; In this code, the game update/calculation rate is locked down so that the game behaves
; the same for different speed of computers, but the graphics always refreshes as fast as possible.
Const c_GameRate=30
period=1000/c_GameRate

; ==============================================================================

overlay = LoadSprite("data\logo.png", 2)
ScaleSprite    overlay, 1, 0.125
PositionEntity overlay, 0, 0, 2
SL_Update()
RenderWorld
Flip

game_create()

; ==============================================================================
;
;                      *******  MAIN GAME LOOP  *******
;
; ==============================================================================
; MAIN GAME LOOP that maintains stable frame rate for all parts of the game
;
theTime=MilliSecs()

While Not KeyHit( 1 ) 

	; Compute how much time has elapsed thru the entire main loop
	Repeat
		elapsedTime=MilliSecs()-theTime
	Until elapsedTime<>0
	
	;how many 'frames' have elapsed	
	elapsedFrames=elapsedTime/period
	
	; If more than 1 frame has elapsed then we have to compute
	; where what the state of the world would have been.
	; This is crucial for slower computers.
	; So this loop goes thru each frame of missed time and computes
	; what the new state of the world should be by simplying
	; calling the users UpdateGameState elapsedFrames times.
	For k=1 To elapsedFrames
	
		; When elapsedFrames is reached, save the state of the world so that tweening
		; can be used to get an accurate interpolation of what the state of the world
		; should be.
		If k=elapsedFrames Then CaptureWorld
		theTime=theTime+period
		
		; Update the game's state (e.g. position of each entity)		
		GameProcess()

		; Update all animation properties- e.g. compute collision checks
		UpdateWorld
	Next

	;Compute the tweening value (a number from 0 to 1)
	; 0 means the state of the world when CaptureWorld was called
	; 1 means the current interpolated state of the world
	; e.g. if elapsedTime is 1 second and desired period is 3 second
	; then 1mod3=1 hence tween=1/period = 1/3 
	; so we interpolate 1/3 distance from the captured point and the present time.
	;
	; if elapsedTime = 5 seconds and desired period is 3 seconds;
	; 1 frame of time would have passed so it would have gone thru
	; the for loop once computing the new frame state
	; Since 5mod3=2 hence tween=2/3 so we tween 2/3 the way between the captured state
	; and the current state.
	tween#=Float(elapsedTime Mod period)/Float(period)


	; Switch to mono mode by pressing 1
	If KeyHit(2)>0 Then
		g_viewMode=1
		SL_SwitchToMono()
	EndIf
	
	; Switch to stereo mode by pressing 2
	If KeyHit(3)>0 Then
		g_viewMode=0
		g_reverseStereo#=1
		SL_SwitchToStereo()
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf

	; Switch to cross eye stereo mode by pressing 3
	If KeyHit(4)>0 Then
		g_viewMode=0
		g_reverseStereo#=-g_reverseStereo#
		SL_SwitchToStereo()
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	
	; Change eye separation by pressing + and -
	If KeyHit(13)>0 Then
		g_eyeSeparation#=g_eyeSeparation#+0.1
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	
	If KeyHit(12) > 0 Then
		g_eyeSeparation#=g_eyeSeparation#-0.1
		If g_eyeSeparation# < 0 Then g_eyeSeparation#=0
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	
	; Adjust stereo cameras and render the world
	SL_Update()
	RenderWorld tween#
	Flip
	
	; F10: Snap a screenshot
	If KeyHit(68) SaveBuffer(FrontBuffer(), "screen.bmp")
	
	; F5/F6: Decrease/increase music volume.
	If KeyHit(63) music_softer()
	If KeyHit(64) music_louder()
Wend 

game_delete()

; ***********************************************************************
; DO YOUR OWN STUFF HERE!!!!!!
Function GameProcess()
	game_update()
	game_state()
End Function

End